//------------------------------------------------------------------------------
//	Command-line parser
//
//	Author: Grzegorz Skowron & Milosz Zdybal
//------------------------------------------------------------------------------

#ifndef __COSKEL_CLPARSER__
#define __COSKEL_CLPARSER__

#include <vector>
#include <string>
#include <cstring>
#include <getopt.h>

//------------------------------------------------------------------------------
//	Class for command line arguments parsing
//
//	Description:
//	This class parses a given command line and allow a programmer to extract
//	desired data. Before any data will be extracted all switches, no-name arguments
//	and parametric arguments have to be registered
//
//	Warning!: Do not extract data in time-critical places of your program. Copy
//	all the necessary data to temporary variables before you get into time-critical
//	section.
//------------------------------------------------------------------------------

extern char *optarg;
extern int optind, opterr, optopt;

//------------------------------------------------------------------------------
struct CLARG
{
	std::string name;
	std::string description;
	std::string value;
	bool is_parametric;
	bool is_required;
	bool is_present;
	bool is_noname;
};

namespace coskel {
	
class clparser
{
	private:
		std::vector<struct CLARG> args;
		std::vector<struct CLARG> noname_args;
		
	public:
		clparser() {}
		
		// Register a noname argument of desired name and description.
		// Description is used to prepare help info and name for addressing.
		// Noname arguments are treated as "required" by default.
		// RETURN: true upon success, false otherwise
		bool register_noname_arg(const std::string &name,const std::string &desc);
		
		// Register an argument of desired name and description. You also need to specify.
		// whether it's parametric and required.
		// RETURN: true upon success, false otherwise
		bool register_arg(const std::string &name,const std::string &desc,bool is_parametric,bool is_required);

		// Parse the command line.
		// RETURN: true upon success, false otherwise (is_required criteria was not met)
		bool parse(int argc,char *argv[]);
	
		// Check for presence of a particular argument
		// RETURN: true if present, false otherwise
		bool check_presence(const std::string &name);
		
		// Returns the value connected with an argument of specified name
		// RETURN: value if success, empty string on failure
		std::string get_value(const std::string &name);
		
		// Equivalent of get_value but for noname arguments
		// RETURN: value if success, empty string on failure
		std::string get_noname_value(const std::string &name);
		
		// Returns information about parameters. Can be used for displaying help information
		std::string get_info(const std::string &progname); 
};	
//------------------------------------------------------------------------------
bool clparser::register_noname_arg(const std::string &name,const std::string &desc)
{
	struct CLARG tmp;
	
	tmp.name = name;
	tmp.description = desc;
	tmp.value.clear();
	tmp.is_parametric = false;
	tmp.is_required = true;
	tmp.is_present = false;
	tmp.is_noname = true;
	
	noname_args.push_back(tmp);
}
//------------------------------------------------------------------------------
bool clparser::register_arg(const std::string &name,const std::string &desc,bool is_parametric,bool is_required)
{
	struct CLARG tmp;
	
	tmp.name = name;
	tmp.description = desc;
	tmp.value.clear();
	tmp.is_parametric = is_parametric;
	tmp.is_required = is_required;
	tmp.is_present = false;
	tmp.is_noname = false;
	
	args.push_back(tmp);
}
//------------------------------------------------------------------------------
bool clparser::parse(int argc,char *argv[])
{
	int opt,c;
	struct option *longopts;
	
	// Alloc enough space to store all options
	longopts = new struct option[args.size()+1];
	if(longopts == NULL) return false;
	
	// Set the content of all option structures
	for(int i=0;i < args.size();i++)
	{
		longopts[i].name = new char[args[i].name.length()+1];
		if(longopts[i].name == NULL)
		{	
			delete [] longopts; // If error - remember to free all previously allocated memory
			return false;
		}
		
		strncpy(const_cast<char*>(longopts[i].name),args[i].name.c_str(),args[i].name.length()); 
		longopts[i].has_arg = args[i].is_parametric;
		longopts[i].flag = NULL;
		longopts[i].val = i;
	}
	memset((void*)&longopts[args.size()],NULL,sizeof(struct option));
	
	// Perform command-line parsing
	while((opt = getopt_long_only(argc,argv,"",longopts,NULL)) != -1)
	{		
		args[opt].is_present = 1;
		if(args[opt].is_parametric && optarg != NULL)
			args[opt].value = std::string(optarg);		
	}
	
	c=0;
	
	// Options are not needed anymore
	for(int i=0;i < args.size();i++) 
		delete [] longopts[i].name;
	delete [] longopts;
	
	// Check the number of provided "noname" arguments
	// They are treated as REQUIRED by default
	if(argc-optind != noname_args.size())
		return false;		
	else
	{
		for(int i=optind;i<argc;i++)
		{
			noname_args[c].is_present = 1;	
			noname_args[c].value = std::string(argv[i]);
			c++;
		}
	}
	
	// Check if all required arguments are present
	for(int i=0;i < args.size();i++)
		if(args[i].is_required == true && args[i].is_present != true)
				return false;
	
	return true;
}
//------------------------------------------------------------------------------	
bool clparser::check_presence(const std::string &name)
{
	for(int i=0;i<args.size();i++)
		if(args[i].name == name) return args[i].is_present;
		
	return false;
}
//------------------------------------------------------------------------------
std::string clparser::get_value(const std::string &name)
{
	for(int i=0;i<args.size();i++)
		if(args[i].name == name) return args[i].value;
		
	return "";
}
//------------------------------------------------------------------------------
std::string clparser::get_noname_value(const std::string &name)
{
	for(int i=0;i<noname_args.size();i++)
		if(noname_args[i].name == name) return noname_args[i].value;
		
	return "";
}
//------------------------------------------------------------------------------	
std::string clparser::get_info(const std::string &progname)
{
	std::string line;

	line = "--------------------------------\nSyntax: " + progname;

	// If there are any optional arguments show [options] text
	for(int i=0;i < args.size();i++)
	{
		if(args[i].is_required == false)
		{
			line += " [options] ";
			break;
		}
	}
	
	// Show all required arguments
	for(int i=0;i < args.size();i++)
	{
		if(args[i].is_required)
		{
			if(args[i].is_parametric == true)
				line += "-" + args[i].name + "=[value]";
			else
				line += "-" + args[i].name;
		}
	}
	
	// Show all "noname" arguments
	for(int i=0;i < noname_args.size();i++)
	{
		line += "<" + noname_args[i].name + "> ";
	}
	line += "\n--------------------------------\n";
	
	// Display info about "noname" arguments
	for(int i=0;i < noname_args.size();i++)
		line += "\t<"+noname_args[i].name+">\n\t\t("+noname_args[i].description+")\n";
	
	line += "\n\n";
	
	// Display info about switches and parametric arguments
	for(int i=0;i < args.size();i++)
	{
			if(args[i].is_parametric == true)
				line += "\t-"+args[i].name+"=<value>";
				
			else if(args[i].is_parametric == false)
				line += "\t-"+args[i].name;
				
			line += "\n\t\t("+args[i].description+")\t";

			if(args[i].is_required)
				line += "!REQUIRED! ";
				
			line += "\n";
	}
	
	line += "\n";

	return line;	
}
//------------------------------------------------------------------------------
	
};

#endif
