//------------------------------------------------------------------------------
//	Math matrix
//
//	Author: Grzegorz Skowron & Milosz Zdybal
//------------------------------------------------------------------------------

#ifndef __COSKEL_MATRIX__
#define __COSKEL_MATRIX__

#include <vector>
#include "coskel_vector.h"

namespace coskel {

//------------------------------------------------------------------------------
//	coskel::matrix<T>
//------------------------------------------------------------------------------

// Warning: Class is designed for storing rectangular matrices!
// Storing triangular matrices with rows of not equal size might
// result in unpredictable behavior. If you want to store a triangular matrix
// in the class better fill the rest with zeros.

// For complex arguments ALWAYS specify -1 for limits if you want to load
// a whole file! (only complex numbers, with real numbers you can take advantage
// of default parameters)

template<class T>
class matrix
{
	private:
		std::vector<coskel::vector<T> > m_vectors;
			
	public:
		
		// Empty constructor for compatibility reasons
		matrix();
		
		// Creates a matrix of "rows" rows and "cols" columns
		// Specifying zeros for dimensions will not create any matrix
		matrix(unsigned int rows,unsigned int cols);
		
		// Copy constructor
		matrix(const matrix<T> &source);
		
		// Copy constructor with limits. type any "beg" AND "end" limits
		// as -1 to copy everything, but better don't do it but simply
		// use the copy constructor above :)
		matrix(const matrix<T> &source,int row_beg,int row_end,int col_beg,int col_end);
			
		// Resizes the matrix to the specified number of rows and columns
		bool resize(unsigned int rows,unsigned int cols);
		
		// Returns the number of rows
		unsigned int rows();		
		
		// Returns the number of columns
		// WARNING!: It uses only the size of the first row in matrix
		// This function should not be used if you store many rows
		// with different number of allocated elements.
		unsigned int cols();
		
		// Loads a matrix from the specified file (skipping all comments starting with #)
		// To read the whole file specify the "beg" and/or "end" limits as -1
		// Otherwise "beg" to "end" numbers are loaded.
		// RETURN: true - upon success, false - otherwise	
		bool load_from_file (const std::string &file_name,int row_beg,int row_end,int col_beg,int col_end);
		
		// Saves the matrix to the specified file
		// To write the whole vector to the file specify the "beg" and/or "end" as -1
		// Otherwise "beg" to "end" numbers are saved.
		// To add some extra information at the top of the file use "extra_info". Otherwise
		// leave it blank ("")
		// RETURN: true - upon success, false - otherwise	
		bool save_to_file (const std::string &file_name,const std::string &extra_info,int row_beg,int row_end,int col_beg,int col_end);
		
		// Returns the matrix as a std::vector of coskel::vector objects
		std::vector<coskel::vector<T> >& get_vector() const;
		
		// Assigns zeros to the all existing matrix elements
		void zero_matrix();
		
		const matrix<T>& operator=(matrix<T> &op);	
		
		// Operators used for addressing rows
		// After you get access to a row you can use another [] operator to
		// get access to an element
		coskel::vector<T>& operator[](unsigned int index);
		coskel::vector<T>& operator[](unsigned int index) const;
		
		// Pushes a new row at the end of the matrix (This is not a fast operation!)
		coskel::matrix<T>& operator<<(const coskel::vector<T> &op);
		
		// Pops (gets and removes) the last row from a matrix (This is not a fast operation!)
		coskel::matrix<T>& operator>>(coskel::vector<T> &op);
};

//------------------------------------------------------------------------------
template<class T>
matrix<T>::matrix()
{}
//------------------------------------------------------------------------------
template<class T>
matrix<T>::matrix(unsigned int rows,unsigned int cols)
{
	if(rows > 0 && cols > 0)
	{
		m_vectors.resize(rows);
		for(unsigned int i=0;i<rows;i++)
			m_vectors[i].resize(cols);
	}
}
//------------------------------------------------------------------------------
template<class T>
matrix<T>::matrix(const matrix<T> &source)
{
	m_vectors.resize(source.m_vectors.size());
	for(unsigned int i=0;i<source.m_vectors.size();i++)
		m_vectors[i] = source.m_vectors[i];
}
//------------------------------------------------------------------------------
template<class T>
matrix<T>::matrix(const matrix<T> &source,int row_beg,int row_end,int col_beg,int col_end)
{
	if(row_end >= row_beg && col_end >= col_beg)
	{
		if((row_beg < 0 || row_end < 0) && (col_beg < 0 || col_end < 0))
		{
			row_beg = 0;
			col_beg = 0;
			row_end = source.m_vectirs.size()-1;
			col_end = source.m_data.size()-1;
		}
		if(row_beg < 0) row_beg = 0;
		if(col_beg < 0) col_beg = 0;
		if(row_beg >= source.rows()) row_beg = source.rows()-1;
		if(col_beg >= source.cols()) col_beg = source.cols()-1;	
		
		if(row_end < 0) row_end = 0;
		if(col_end < 0) col_end = 0;
		if(row_end >= source.rows()) row_end = source.rows()-1;
		if(col_end >= source.cols()) col_end = source.cols()-1;		
		
		m_vectors.resize(std::min(source.m_vectors.size(),row_end-row_beg+1));
		for(unsigned int i=0;i<std::min(source.m_vectors.size(),row_end-row_beg+1);i++)
		{
			m_vectors[i].resize(std::min(source.m_vectors[i].size(),col_end-col_beg+1));
			for(unsigned int j=0;j<std::min(source.m_vectors[i].size(),col_end-col_beg+1);j++)
				m_vectors[i][j] = source.m_vectors[i][col_beg+j]; 
		}
	}	
}
//------------------------------------------------------------------------------
template<class T>
bool matrix<T>::resize(unsigned int rows,unsigned int cols)
{
	m_vectors.resize(rows);
	if(m_vectors.size() != rows) return false;
	
	for(unsigned int i=0;i<rows;i++)
	{
		m_vectors[i].resize(cols);
		if(m_vectors[i].size() != cols) return false;
	}
		
	return true;	
}
//------------------------------------------------------------------------------
template<class T>
unsigned int matrix<T>::rows()
{
	return m_vectors.size();
}
//------------------------------------------------------------------------------		
template<class T>
unsigned int matrix<T>::cols()
{
	if(m_vectors.size() > 0)
		return m_vectors[0].size();
	else
		return 0;
}
//------------------------------------------------------------------------------
template<class T>
std::vector<coskel::vector<T> >& matrix<T>::get_vector() const
{
	return m_vectors;
}
//------------------------------------------------------------------------------
template<class T>
void matrix<T>::zero_matrix()
{
	for(unsigned int i=0;i<m_vectors.size();i++)
		m_vectors[i].zero_vector();
}
//------------------------------------------------------------------------------
template<class T>
const matrix<T>& matrix<T>::operator=(matrix<T> &op)
{
	m_vectors.resize(op.m_vectors.size());
	for(unsigned int i=0;i<m_vectors.size();i++)
		m_vectors[i] = op.m_vectors[i];
}
//------------------------------------------------------------------------------		
template<class T>
coskel::vector<T>& matrix<T>::operator[](unsigned int index)
{
	return m_vectors[index];
}
//------------------------------------------------------------------------------
template<class T>
coskel::vector<T>& matrix<T>::operator[](unsigned int index) const
{
	return m_vectors[index];
}
//------------------------------------------------------------------------------
template<class T>
coskel::matrix<T>& matrix<T>::operator<<(const coskel::vector<T> &op)
{
	m_vectors.push_back(op);
	return *this;
}
//------------------------------------------------------------------------------
template<class T>
coskel::matrix<T>& matrix<T>::operator>>(coskel::vector<T> &op)
{
	op = m_vectors.back();
	m_vectors.pop_back();
	return *this;
}
//------------------------------------------------------------------------------	
template<class T>
bool matrix<T>::load_from_file (const std::string &file_name,int row_beg = -1,int row_end = -1,int col_beg = -1,int col_end = -1)
{
	int pos = 0;
	vector<T> tmp;
	std::ifstream infile(file_name.c_str());
	
	if(!infile.good()) return false;
	
	if(row_beg > row_end || col_beg > col_end || ((row_beg < 0 || row_end < 0) && (col_beg < 0 || col_end < 0)))
	{
		while(infile.good())
		{
			tmp.load_line_from_file(infile);
			if(tmp.size() > 0) (*this) << tmp;
		}
	}
	else
	{
		
		while(infile.good())
		{
			tmp.load_line_from_file(infile);
			if(row_beg < 0 || row_end < 0 || (pos >= row_beg && pos <= row_end))
			{
				if((col_beg < 0 || col_end < 0) && tmp.size() > 0)
					(*this) << tmp;
				else
				{
					vector<T> local_tmp(tmp,col_beg,col_end);
					if(local_tmp.size() > 0) (*this) << local_tmp;
				}
			}
			if(tmp.size() > 0) pos++;
		}
	}
	infile.close();
	
	return true;
}
//------------------------------------------------------------------------------
template<class T>
bool matrix<T>::save_to_file (const std::string &file_name,const std::string &extra_info,int row_beg = -1,int row_end = -1,int col_beg = -1,int col_end = -1)
{
	std::ofstream outfile(file_name.c_str());
	
	if(!outfile.good()) return false;
	
	if(!extra_info.empty()) outfile << extra_info << std::endl;
	
	if(row_beg > row_end || col_beg > col_end || ((row_beg < 0 || row_end < 0) && (col_beg < 0 || col_end < 0)))
	{
		for(int i=0;i<m_vectors.size();i++)
		{
			for(int j=0;j<m_vectors[i].size();j++)
				outfile << m_vectors[i][j] << '\t';
			outfile << std::endl;
		}
	}
	else
	{		
		if(row_beg < 0) row_beg = 0;
		if(row_end < 0) row_end = 0;

		if(col_beg < 0) col_beg = 0;
		if(col_end < 0) col_end = 0;
		
		if(row_beg >= rows()) row_beg = rows()-1;
		if(row_end >= rows()) row_end = rows()-1;

		if(col_beg >= cols()) col_beg = cols()-1;
		if(col_end >= cols()) col_end = cols()-1;	
		
		for(int i=row_beg;i<=row_end;i++)
		{
			for(int j=col_beg;j<=col_end && j < m_vectors[i].size();j++)
				outfile << m_vectors[i][j] << '\t';
			outfile << std::endl;
		}
		
	}
	
	outfile.close();
	
	return true;
}
//------------------------------------------------------------------------------
template<>
bool matrix<std::complex<double> >::save_to_file (const std::string &file_name,const std::string &extra_info,int row_beg,int row_end,int col_beg,int col_end)
{
	std::ofstream outfile(file_name.c_str());
	
	if(!outfile.good()) return false;
	
	if(!extra_info.empty()) outfile << extra_info << std::endl;
	
	if(row_beg > row_end || col_beg > col_end || ((row_beg < 0 || row_end < 0) && (col_beg < 0 || col_end < 0)))
	{
		for(int i=0;i<m_vectors.size();i++)
		{
			for(int j=0;j<m_vectors[i].size();j++)
				outfile << complex_to_string(m_vectors[i][j]) << '\t';
			outfile << std::endl;
		}
	}
	else
	{		
		if(row_beg < 0) row_beg = 0;
		if(row_end < 0) row_end = 0;

		if(col_beg < 0) col_beg = 0;
		if(col_end < 0) col_end = 0;
		
		if(row_beg >= rows()) row_beg = rows()-1;
		if(row_end >= rows()) row_end = rows()-1;

		if(col_beg >= cols()) col_beg = cols()-1;
		if(col_end >= cols()) col_end = cols()-1;	
		
		for(int i=row_beg;i<=row_end;i++)
		{
			for(int j=col_beg;j<=col_end && j < m_vectors[i].size();j++)
				outfile << complex_to_string(m_vectors[i][j]) << '\t';
			outfile << std::endl;
		}
		
	}
	
	outfile.close();
	
	return true;
}

};

#endif
