//------------------------------------------------------------------------------
//	Math vector
//
//	Author: Grzegorz Skowron & Milosz Zdybal
//------------------------------------------------------------------------------

#ifndef __COSKEL_VECTOR__
#define __COSKEL_VECTOR__

#include <complex>
#include <valarray>
#include <fstream>
#include <cstdlib>
#include "coskel_utils.h"

namespace coskel {

//------------------------------------------------------------------------------
//	coskel::vector<T>
//------------------------------------------------------------------------------

//	Desription:
//
//	The class can be used for storing values of any numerical type
//	including complex<double> and for safety purposes ONLY complex<double>

// For complex arguments ALWAYS specify -1 for limits if you want to load
// a whole file! (only complex numbers, with real numbers you can take advantage
// of default parameters)

template<class T> class matrix;

template<class T>
class vector
{
	friend class matrix<T>;
	
	private:
		std::valarray<T> m_data;
		
		// Not for explicit use
		T to_current_type(const std::string &str);
		
		bool load_line_from_file(std::ifstream &input);
		bool load_all_from_file(std::ifstream &input);
			
	public:
		
		// These are self explanatory :)
		vector();
		vector(unsigned int size);
		vector(const vector<T> &source);
		vector(const vector<T> &source,int beg,int end); // including "beg" and "end" indexes
		
		// Resizes the vector to the desired size
		// RETURN: true - upon success, false - otherwise		
		bool resize(unsigned int newsize);
		
		// Returns the size of the vector
		unsigned int size() const;
		
		// Loads a vector from the specified file (skipping all comments starting with #)
		// To read the whole file specify the "beg" and/or "end" as -1
		// Otherwise "beg" to "end" numbers are loaded.
		// RETURN: true - upon success, false - otherwise			
		bool load_from_file (const std::string &file_name,int beg,int end);
		
		// Saves the vector to the specified file
		// To write the whole vector to the file specify the "beg" and/or "end" as -1
		// Otherwise "beg" to "end" numbers are saved.
		// To add some extra information at the top of the file use "extra_info". Otherwise
		// leave it blank ("")
		// RETURN: true - upon success, false - otherwise	 
		bool save_to_file (const std::string &file_name,const std::string &extra_info,int beg,int end);
		
		// This is the way to get to the internal valarray object :)
		std::valarray<T>& get_valarray() const;
		
		// Assigns zero to each element of the vector
		void zero_vector();
		
		// Makes a complex<double> vector of size N/2 from the
		// current vector of size N of any non-complex values
		vector<std::complex<double> > to_complex_vector() const;
		
		// Assigns one vector to another
		const vector<T>& operator=(const vector<T> &op);
		
		// Gets to any element by index
		T& operator[](unsigned int index);
		T operator[](unsigned int index) const;
		
		// Puts one element on the back of the vector. (This is not a fast operation!)
		vector<T>& operator<<(const T &op);
		
		// Pops one element from the back of the vector (removing the original one; This is not a fast operation!)
		vector<T>& operator>>(T &op);
		
};

//------------------------------------------------------------------------------
template<class T>
vector<T>::vector() 
{
	resize(0);
}
//------------------------------------------------------------------------------
template<class T>
vector<T>::vector(unsigned int size)
{
	resize(size);
}
//------------------------------------------------------------------------------
template<class T>
vector<T>::vector(const vector<T> &source,int beg,int end)
{	
	if(end >= beg)
	{
		// Ensure that negative and too high values will be corrected
		
		if(beg >= (int)source.size()) beg = (int)source.m_data.size()-1;
		if(beg < 0) beg = 0;
		
		if(end >= (int)source.size()) end = (int)source.m_data.size()-1;
		if(end < 0) end = 0;
			
		// Copy values from the source vector
		resize(std::min(source.size(),(unsigned int)(end-beg+1)));
		for(unsigned int i=0;i<std::min(source.size(),(unsigned int)(end-beg+1));i++)
			m_data[i] = source.m_data[beg+i];
	}
}
//------------------------------------------------------------------------------
template<class T>
vector<T>::vector(const vector<T> &source)
{
	m_data.resize(source.size());
	for(unsigned int i=0;i<size();i++)
		m_data[i] = source.m_data[i];
}
//------------------------------------------------------------------------------
template<class T>
bool vector<T>::resize(unsigned int newsize)
{
	std::valarray<T> tmp(newsize);
	
	if(tmp.size() != newsize) return false;
	
	// Copy data to a temporary container
	for(unsigned int i=0;i<std::min(newsize,size());i++) 
		tmp[i] = m_data[i];
	
	// Resize the vector
	m_data.resize(newsize);
	if(m_data.size() != newsize) return false;
	
	// Copy data again to our container
	for(unsigned int i=0;i<size();i++) 
		m_data[i] = tmp[i];
	
	return true;
}
//------------------------------------------------------------------------------
template<class T>
unsigned int vector<T>::size() const
{
	return m_data.size();	
}
//------------------------------------------------------------------------------
template<class T>
std::valarray<T>& vector<T>::get_valarray() const
{
	return m_data;
}
//------------------------------------------------------------------------------
template<class T>
void vector<T>::zero_vector()
{
		for(unsigned int i=0;i<size();i++)
			m_data[i] = 0;	
}
//------------------------------------------------------------------------------
template<class T>
const vector<T>& vector<T>::operator=(const vector<T> &op)
{
	m_data.resize(op.size());
	for(unsigned int i=0;i<size();i++)
		m_data[i] = op.m_data[i];
	return *this;
}
//------------------------------------------------------------------------------
template<class T>
T& vector<T>::operator[](unsigned int index)
{
	return m_data[index];
}
//------------------------------------------------------------------------------
template<class T>
T vector<T>::operator[](unsigned int index) const
{
	return m_data[index];
}
//------------------------------------------------------------------------------
template<class T>
vector<T>& vector<T>::operator<<(const T &op)
{
	resize(size()+1);
	m_data[size()-1] = op;
	return *this;
}
//------------------------------------------------------------------------------
template<class T>
vector<T>& vector<T>::operator>>(T &op)
{
	op = m_data[size()-1];
	resize(size()-1);
	return *this;
}
//------------------------------------------------------------------------------
template<class T>
vector<std::complex<double> > vector<T>::to_complex_vector() const
{
	vector<std::complex<double> > tmp(size()/2);
	
	for(unsigned int i=0;i<size()/2;i++) 
		tmp[i] = std::complex<double>(m_data[2*i],m_data[2*i+1]);
	
	return tmp;
}
//------------------------------------------------------------------------------
template<class T>
bool vector<T>::load_all_from_file(std::ifstream &input)
{
	unsigned int num_count = 0;
	vector<T> tmp;
	
	if(!input.good()) return false;
	
	resize(0);
	
	while(tmp.load_line_from_file(input))
	{
		if(tmp.size() > 0)
		{
			if(!resize(size()+tmp.size())) return false;
			for(unsigned int i=0;i<tmp.size();i++)
				m_data[num_count++]= tmp[i];

		}
	}
	return true;
}
//------------------------------------------------------------------------------
template<class T>
T vector<T>::to_current_type(const std::string &str)
{ 
	return atof(str.c_str());
}
//------------------------------------------------------------------------------
template<>
std::complex<double> vector<std::complex<double> >::to_current_type(const std::string &str)
{ 
	return string_to_complex(str);
}
//------------------------------------------------------------------------------
template<class T>
bool vector<T>::load_line_from_file(std::ifstream &input)
{
	char c;
	unsigned int num_count = 0;
	std::string line;
	std::string token;
	size_t pos;
	
	if(!input.good()) return false;
	
	// Read a line
	while(input.good())
	{
		input.get(c);
		line += c;
		if(c == '\n' || c == '\r') break;
	}
	
	// Find comments and erase everything from a comment mark to the end of line
	if((pos = line.find("#")) != std::string::npos)
		line.erase(pos);
	
	// Prepare a stream for easier data extraction
	std::istringstream line_stream(line);
	
	if(!resize(1)) return false;
	
	// Extract tokens
	while(line_stream >> token)
	{
		if(token.length() == 1 && isalnum(token[0]) == 0) continue;
		if(num_count >= size()) if(!resize(2 * size())) return false;
		m_data[num_count] = to_current_type(token);
		num_count++;
	}
	
	// Scale down our (probably) oversized vector
	if(!resize(num_count)) return false;
		
	return true;
}
//------------------------------------------------------------------------------
template<class T>
bool vector<T>::load_from_file(const std::string &file_name,int beg = -1,int end = -1)
{
	std::ifstream infile(file_name.c_str());
	if(!infile.good()) return false;
	
	if(beg > end || beg < 0 || end < 0)
	{
		
		// If limits are improper then load the complete file
		if(this->load_all_from_file(infile))
		{
			infile.close();
			return true;	
		}
		else
		{
			infile.close();
			return false;	
		}
	}
	else // Otherwise load line by line putting everything interesting to our vector
	{
		vector<T> tmp;
		if(!tmp.load_all_from_file(infile)) return false;
		
		// Correct positive but improper (exceeding) limits
		if(beg >= (int)tmp.size()) beg = (int)tmp.m_data.size()-1;
		if(end >= (int)tmp.size()) end = (int)tmp.m_data.size()-1;
		
		if(!resize(std::min((unsigned int)(end-beg+1),tmp.size()))) return false;
		for(unsigned int i=0;i < std::min((unsigned int)(end-beg+1),tmp.size());i++)
			m_data[i] = tmp[beg+i];
	}
	infile.close();
	
	return true;
}
//------------------------------------------------------------------------------
template<>
bool vector<std::complex<double> >::save_to_file(const std::string &file_name,const std::string &extra_info,int beg,int end)
{
	std::ofstream outfile(file_name.c_str());
	
	if(!outfile.good()) return false;
	
	// Save extra information at the beginning of the specified file
	if(!extra_info.empty()) outfile << extra_info << std::endl;
	
	if(beg > end || beg < 0 || end < 0)
	{
		for(unsigned int i=0;i < size();i++)
			outfile << complex_to_string(m_data[i]) << std::endl;
	}
	else
	{
		if(beg >= (int)size()) beg = (int)m_data.size()-1;
		if(end >= (int)size()) end = (int)m_data.size()-1;
		
		for(unsigned int i=0;i < std::min((unsigned int)(end-beg+1),size());i++)
			outfile << complex_to_string(m_data[i]) << std::endl;
	}
	outfile.close();
	
	return true;
}
//------------------------------------------------------------------------------
template<class T>
bool vector<T>::save_to_file(const std::string &file_name,const std::string &extra_info,int beg = -1,int end = -1)
{
	std::ofstream outfile(file_name.c_str());
	
	if(!outfile.good()) return false;
	
	if(!extra_info.empty()) outfile << extra_info << std::endl;
	
	if(beg > end || beg < 0 || end < 0)
	{
		for(unsigned int i=0;i < size();i++)
			outfile << m_data[i] << std::endl;
	}
	else
	{
		if(beg >= (int)size()) beg = (int)m_data.size()-1;
		if(end >= (int)size()) end = (int)m_data.size()-1;
		
		for(unsigned int i=0;i < std::min((unsigned int)(end-beg+1),size());i++)
			outfile << m_data[beg+i] << std::endl;
	}
	outfile.close();
	
	return true;
}
//------------------------------------------------------------------------------

};

#endif
