//------------------------------------------------------------------------------
//	Configuration parser
//
//	Author: Grzegorz Skowron & Milosz Zdybal
//------------------------------------------------------------------------------

#ifndef __COSKEL_CONFPARSER__
#define __COSKEL_CONFPARSER__

#include <map>
#include <string>
#include <fstream>

namespace coskel {

//------------------------------------------------------------------------------
//	Simple class for parsing configuration files 
//
//	Description:
//	Comments starting with "#" are accepted in any place.
//	Parameters are key=value couples. Spaces before key, after value
//	and around = are accepted and omitted
//
//	Warning!: All entries containing key or value consisting of whitespaces only
//	will be ommited! Both key and value have to comprise at least of one alphanumeric character
//
//	Advice: Use configuration files to set all necessary numerical values
//	instead of command line parameters. It's faster and more reliable.
//------------------------------------------------------------------------------

class confparser
{
	private:
		std::map<std::string,std::string> items;
	
	public:
		
		// Default constructor for compatibility reson
		confparser(){}
		
		// Constructs an object and loads the file
		confparser(const std::string &filename) { load_file(filename); }
	
		// Load and parse file
		// Returns true upon success, false otherwise
		// Better use this function instead of the constructor
		// because you can check the operation status
		bool load_file(const std::string &filename);
		
		// Get the value of a key element.
		// If the specified key doesn't exist, an empty string ("") is returned
		// This uses map so it's O(lg n)
		std::string operator[](const std::string &key);
		
		// Returns some overall information about the config options
		// Used mainly for coskell::prepare_info function purpose.
		std::string get_info();
};

//------------------------------------------------------------------------------
bool confparser::load_file(const std::string &filename)
{
	char c;
	int eqpos;
	int beg,end;
	std::string line;
	std::string key,value;
	size_t pos;
	
	std::ifstream input(filename.c_str());
	if(!input.good()) return false;
	
	while(input.good())
	{
		line.clear();
		eqpos = -1;
		key.clear();
		value.clear();
		
		// Read a line
		while(input.good())
		{
			input.get(c);
			if(c == '\n' || c == '\r') break;
			line += c;
		}
	
		// Find comments and erase everything from a comment mark to the end of line
		if((pos = line.find("#")) != std::string::npos)
			line.erase(pos);
			
		if(line.length() <= 0) continue;
			
		// Find the '=' character and split into key and value
		for(int i=0;i<line.length();i++)
			if(line[i] == '=') eqpos = i;
		
		if(eqpos != -1 && eqpos != 0 && eqpos != line.length()-1)
		{			
			key = line.substr(0,eqpos);
			value = line.substr(eqpos+1);			
			
			beg = key.find_first_not_of(" \t");
			end = key.find_last_not_of(" \t");
			
			if(beg < 0 || end < 0) continue;
			
			key = key.substr(beg,end-beg+1);
			
			beg = value.find_first_not_of(" \t");
			end = value.find_last_not_of(" \t");
			
			if(beg < 0 || end < 0) continue;
			
			value = value.substr(beg,end-beg+1);
			
			if(key.length() > 0 && value.length() > 0)
				items.insert(std::pair<std::string,std::string>(key,value));
		}	
	}
		
	return true;
}
//------------------------------------------------------------------------------
std::string confparser::operator[](const std::string &key)
{
	if(items.find(key) != items.end())
		return items[key];
	else
		return std::string("");
}
//------------------------------------------------------------------------------
std::string confparser::get_info()
{
	std::string line="";
	
	std::map<std::string,std::string>::iterator it;
	
	for(it = items.begin();it != items.end();it++)
	{
		(*it).first;
		(*it).second;
		
		line += "##\t@ "+ (*it).first + " = " + (*it).second + "\n";
	}
	
	return line;
}
//------------------------------------------------------------------------------
};

#endif

